unit hiScope; {   ver 2.00 }
//
// purpose: Scope,
(*
  TScope Properties:
     Active           Starts/Stops scope
     Color            Backgroundcolor
     Gridcolor        Grid mask color
     Gridsize         Size of grid mask in pixels
     Height           Scopes height in pixels
     Interval         Scroll speed in 1/100's seconds
     LineColor        Scope dataline color
     Position         Dataline value (range 0-100)
     Width            Scopes width in pixels
     Basecolor        Color of baseline
     Baseline         Baseline value (range 0-100)

  Methods:
     Redraw           Clears the control and redraws grid
*)
interface

uses Windows,Messages,Kol,Share,Win;

type
 THIScope = class(THIWin)
  private
    fDrawBuffer : PBitmap;
    fDrawTimer  : PMMTimer;
    fActive     : boolean;
    fBaseColor,           { Baseline color }
    SColor,               { Background color }
    fGridColor,           { Grid line color }
    fLineColor : TColor;  { Position line color }
    fBaseLine,
    fGridSize,
    fPosition,            { Value to plot }
    fInterval  : integer; { Update speed in 1/10 seconds }
    Oldpos, PrevPos : integer;
    CalcBase, Counter : integer;

    procedure Redraw;
    procedure Paint;
    procedure UpdateScope(Sender:PObj);
    procedure _OnClick( Sender: PObj );
    
   public

    _event_OnClick: THI_Event; 
    _data_Position: THI_Event;

    property _prop_Interval  : Integer  write fInterval;
    property _prop_GridSize  : Integer  write fGridSize;
    property _prop_Baseline  : Integer  write fBaseline;    
    property _prop_FonColor  : TColor   write SColor;
    property _prop_BaseColor : TColor   write fBaseColor;
    property _prop_GridColor : TColor   write fGridColor;
    property _prop_LineColor : TColor   write fLineColor;

    procedure Init; override;
    destructor Destroy; override;

    procedure _work_doActive(var _Data:TData; Index:word);
    
    procedure _work_doInterval(var _Data:TData; Index:word);
    procedure _work_doGridSize(var _Data:TData; Index:word);
    procedure _work_doBaseline(var _Data:TData; Index:word);    

    procedure _work_doFonColor(var _Data:TData; Index:word);
    procedure _work_doBaseColor(var _Data:TData; Index:word);
    procedure _work_doGridColor(var _Data:TData; Index:word);    
    procedure _work_doLineColor(var _Data:TData; Index:word);

  end;

implementation

function WndProcScope(Sender: PControl; var Msg: TMsg; var Rslt: Integer): Boolean;
var   P:TPaintSTruct;
      fControl: THIScope;
begin
   Result:=False;
   fControl := THIScope(Sender.Tag); 
   if (msg.Message = WM_PAINT) then begin
      BeginPaint(Sender.Handle,P);
         fControl.Paint;
      EndPaint(Sender.Handle,P);
      Result:=True;
      Rslt := 0;
   end else if (msg.Message = WM_SIZE) and ((fControl.fDrawBuffer.width <> Sender.Width) or
               (fControl.fDrawBuffer.height <> Sender.Height)) then begin
      fControl.fDrawBuffer.width := Sender.Width;
      fControl.fDrawBuffer.height := Sender.Height;
      fControl.Redraw;
   end 
end;

procedure ThiScope.Init;
begin
   Control := NewPaintbox(FParent);
   Control.Tag := longint(Self);
   Control.Attachproc(WndprocScope);
   Control.OnClick := _OnClick;

   fDrawBuffer := NewDIBBitmap(Control.Width,Control.Height,pf32bit);
   if fGridSize < 2 then fGridSize := 2;  
   fGridSize := (fGridSize div 2)*2;

   fDrawTimer          := NewMMTimer(500);
   fDrawTimer.Enabled  := FALSE;
   fDrawTimer.Interval := 500;
   fDrawTimer.Resolution := 1; 
   fDrawTimer.OnTimer := UpdateScope;      

inherited;
   Redraw;
end;

destructor ThiScope.Destroy;
begin
   fDrawBuffer.free;
   fDrawTimer.Free;
   inherited;
end;

procedure ThiScope._OnClick;
begin
   _hi_OnEvent(_event_OnClick);
end;
 
procedure ThiScope.Redraw;{ Redraw control, re-calculate grid etc }
var   a : integer;
begin
   fDrawTimer.Enabled  := FALSE;
   with Control{$ifndef F_P}^{$endif} do begin
      CalcBase := (Height - Round(Height/100*FBaseline));
      with fDrawBuffer.Canvas{$ifndef F_P}^{$endif} do begin
         Brush.Color := SColor;
         Brush.BrushStyle := bsSolid;
         SelectObject(Handle,Brush.Handle);
         Windows.FillRect(Handle,MakeRect(0,0,Width,Height),Brush.Handle);
      end;
      with Control.Canvas{$ifndef F_P}^{$endif} do begin
         Brush.Color := SColor;
         Brush.BrushStyle := bsSolid;
         SelectObject(Handle,Brush.Handle);
         Windows.FillRect(Handle,MakeRect(0,0,Width,Height),Brush.Handle);
         Pen.PenStyle := psSolid;
         Pen.Color := FGridColor;
         Pen.PenWidth := 1;
         { Vertical lines }
         a := Width;
         while a > 0 do begin
            MoveTo(a-1,0);
            LineTo(a-1,Height);
            dec(a,FGridSize);
         end;
         { Horizontal lines - above Baseline }
         a := CalcBase;
         while a < height do begin
            inc(a,FGridSize);
            MoveTo(0,a);
            LineTo(Width,a);
         end;
         { Horizontal lines - below Baseline }
         a := CalcBase;
         while a > 0 do begin
            Dec(a,FGridSize);
            MoveTo(0,a);
            LineTo(Width,a);
         end;
         { Baseline }
         Pen.Color := FBaseColor;
         MoveTo(0,CalcBase);
         LineTo(Width,CalcBase);
         { Start new position-line on baseline... }
         OldPos   := CalcBase;
         PrevPos  := CalcBase;
         Counter := 1;
      end;
   end;
   fDrawTimer.Enabled := FActive;
end;

procedure ThiScope.UpdateScope;
begin
   Paint;
end;

procedure ThiScope.Paint;
{ All drawings is performed on in the drawbuffer to speed up
  proceedings and eliminate flicker. The Paint procedure merely
  copies the contents of the drawbuffer.
}
var   a: integer;
      hdcMem:HDC;
      hdcBmp:HBITMAP;

begin
   fPosition := ToIntegerEvent(_data_Position);
   with Control{$ifndef F_P}^{$endif} do begin
      hdcMem:= CreateCompatibleDC(0);
      hdcBmp:= CreateCompatibleBitmap(Canvas.Handle,Width,Height);
      SelectObject(hdcMem, hdcBmp);

      with fDrawBuffer.Canvas{$ifndef F_P}^{$endif} do begin
         { Copy bitmap leftwards }
         if fDrawTimer.Enabled then begin //   ,   
            CopyRect(MakeRect(0,0,Width-2,Height),fDrawBuffer.Canvas,MakeRect(2,0,Width,Height));
            { Draw new area }
            Pen.Color := SColor;
            Pen.PenWidth := 2;
            MoveTo(Width-1,0);
            LineTo(Width-1,Height);
            { Draw position for line }
            Pen.Color := FLineColor;
            Pen.PenWidth := 1;
            a := Height-round(Height/100*fPosition);
            MoveTo(Width-4,OldPos);
            LineTo(Width-2,PrevPos);
            LineTo(Width-0,a);
            OldPos := PrevPos;
            PrevPos := a;
         end;
         { Vertical lines }
         Canvas.Pen.Color := FGridColor;
         Canvas.Pen.PenWidth := 1;
         SelectObject(hdcMem,Canvas.Pen.Handle);
         a := Width;
         while a > 0 do begin
            Windows.MoveToEX(hdcMem,a-1,0,nil);
            Windows.LineTo(hdcMem,a-1,Height);
            dec(a,FGridSize);
         end;
         { Horizontal lines - above Baseline }
         a := CalcBase;
         while a < height do begin
            inc(a,FGridSize);
            Windows.MoveToEX(hdcMem,0,a,nil);
            Windows.LineTo(hdcMem,Width,a);
         end;
         { Horizontal lines - below Baseline }
         a := CalcBase;
         while a > 0 do begin
            Dec(a,FGridSize);
            Windows.MoveToEX(hdcMem,0,a,nil);
            Windows.LineTo(hdcMem,Width,a);
         end;

         BitBlt(hdcMem,2,0,Width-2,Height,fDrawBuffer.Canvas.Handle,2,0,SRCPAINT);

         { Baseline }
         Canvas.Pen.Color := FBaseColor;
         SelectObject(hdcMem,Canvas.Pen.Handle);
         Windows.MoveToEX(hdcMem,0,CalcBase,nil);
         Windows.LineTo(hdcMem,Width,CalcBase);

         BitBlt(Control.Canvas.Handle,0,0,Width,Height,hdcMem,0,0,SRCCOPY);
      end;
   end;
   DeleteDC(hdcMem);
   DeleteObject(hdcBmp);
end;

Procedure THIScope._work_doActive;
begin
   FActive := ReadBool(_Data);
   fDrawTimer.Enabled := FALSE;
   fDrawTimer.Interval := fInterval*10;
   fDrawTimer.Enabled := FActive;
end;

procedure THIScope._work_doInterval(var _Data:TData; Index:word);
begin
   fInterval := ToInteger(_Data);
   fDrawTimer.Enabled := FALSE;
   fDrawTimer.Interval := fInterval*10;
   fDrawTimer.Enabled := FActive;
end;

procedure THIScope._work_doGridSize(var _Data:TData; Index:word);
begin
   fGridSize := (ToInteger(_Data) div 2)*2;
   if fGridSize < 2 then fGridSize := 2;  
   fGridSize := (fGridSize div 2)*2;
   Redraw;
end;

procedure THIScope._work_doBaseline(var _Data:TData; Index:word);    
begin
   fBaseLine := ToInteger(_Data);
   Redraw;
end;

procedure THIScope._work_doFonColor(var _Data:TData; Index:word);
begin
   SColor := ToInteger(_Data);
   Paint;
end;

procedure THIScope._work_doBaseColor(var _Data:TData; Index:word);
begin
   FBaseColor := ToInteger(_Data);
   Paint;
end;

procedure THIScope._work_doGridColor(var _Data:TData; Index:word);    
begin
   FGridColor := ToInteger(_Data);
   Paint;
end;

procedure THIScope._work_doLineColor(var _Data:TData; Index:word);
begin
   FLineColor := ToInteger(_Data);
   Paint;
end;

end.